@@ -17,14 +17,14 @@
 /**
  * Version details
  *
- * @package    block
- * @subpackage news_items
+ * @package    block_news_items
  * @copyright  1999 onwards Martin Dougiamas (http://dougiamas.com)
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
 defined('MOODLE_INTERNAL') || die();
 
-$plugin->version   = 2012112900;        // The current plugin version (Date: YYYYMMDDXX)
-$plugin->requires  = 2012112900;        // Requires this Moodle version
+$plugin->version   = 2019052000;         // The current plugin version (Date: YYYYMMDDXX)
+$plugin->requires  = 2019051100;         // Requires this Moodle version
 $plugin->component = 'block_news_items'; // Full name of the plugin (used for diagnostics)
+$plugin->dependencies = array('mod_forum' => 2019051100);
