@@ -24,12 +24,9 @@
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
-require_once($CFG->libdir.'/eventslib.php');
-
 defined('MOODLE_INTERNAL') || die();
 
 // File areas for file submission assignment.
-define('ASSIGNSUBMISSION_FILE_MAXFILES', 20);
 define('ASSIGNSUBMISSION_FILE_MAXSUMMARYFILES', 5);
 define('ASSIGNSUBMISSION_FILE_FILEAREA', 'submission_files');
 
@@ -70,12 +67,20 @@ class assign_submission_file extends assign_submission_plugin {
     public function get_settings(MoodleQuickForm $mform) {
         global $CFG, $COURSE;
 
-        $defaultmaxfilesubmissions = $this->get_config('maxfilesubmissions');
-        $defaultmaxsubmissionsizebytes = $this->get_config('maxsubmissionsizebytes');
+        if ($this->assignment->has_instance()) {
+            $defaultmaxfilesubmissions = $this->get_config('maxfilesubmissions');
+            $defaultmaxsubmissionsizebytes = $this->get_config('maxsubmissionsizebytes');
+            $defaultfiletypes = $this->get_config('filetypeslist');
+        } else {
+            $defaultmaxfilesubmissions = get_config('assignsubmission_file', 'maxfiles');
+            $defaultmaxsubmissionsizebytes = get_config('assignsubmission_file', 'maxbytes');
+            $defaultfiletypes = get_config('assignsubmission_file', 'filetypes');
+        }
+        $defaultfiletypes = (string)$defaultfiletypes;
 
         $settings = array();
         $options = array();
-        for ($i = 1; $i <= ASSIGNSUBMISSION_FILE_MAXFILES; $i++) {
+        for ($i = 1; $i <= get_config('assignsubmission_file', 'maxfiles'); $i++) {
             $options[$i] = $i;
         }
 
@@ -85,7 +90,7 @@ class assign_submission_file extends assign_submission_plugin {
                               'maxfilessubmission',
                               'assignsubmission_file');
         $mform->setDefault('assignsubmission_file_maxfiles', $defaultmaxfilesubmissions);
-        $mform->disabledIf('assignsubmission_file_maxfiles', 'assignsubmission_file_enabled', 'notchecked');
+        $mform->hideIf('assignsubmission_file_maxfiles', 'assignsubmission_file_enabled', 'notchecked');
 
         $choices = get_max_upload_sizes($CFG->maxbytes,
                                         $COURSE->maxbytes,
@@ -103,9 +108,15 @@ class assign_submission_file extends assign_submission_plugin {
                               'maximumsubmissionsize',
                               'assignsubmission_file');
         $mform->setDefault('assignsubmission_file_maxsizebytes', $defaultmaxsubmissionsizebytes);
-        $mform->disabledIf('assignsubmission_file_maxsizebytes',
+        $mform->hideIf('assignsubmission_file_maxsizebytes',
                            'assignsubmission_file_enabled',
                            'notchecked');
+
+        $name = get_string('acceptedfiletypes', 'assignsubmission_file');
+        $mform->addElement('filetypes', 'assignsubmission_file_filetypes', $name);
+        $mform->addHelpButton('assignsubmission_file_filetypes', 'acceptedfiletypes', 'assignsubmission_file');
+        $mform->setDefault('assignsubmission_file_filetypes', $defaultfiletypes);
+        $mform->hideIf('assignsubmission_file_filetypes', 'assignsubmission_file_enabled', 'notchecked');
     }
 
     /**
@@ -117,6 +128,13 @@ class assign_submission_file extends assign_submission_plugin {
     public function save_settings(stdClass $data) {
         $this->set_config('maxfilesubmissions', $data->assignsubmission_file_maxfiles);
         $this->set_config('maxsubmissionsizebytes', $data->assignsubmission_file_maxsizebytes);
+
+        if (!empty($data->assignsubmission_file_filetypes)) {
+            $this->set_config('filetypeslist', $data->assignsubmission_file_filetypes);
+        } else {
+            $this->set_config('filetypeslist', '');
+        }
+
         return true;
     }
 
@@ -126,11 +144,15 @@ class assign_submission_file extends assign_submission_plugin {
      * @return array
      */
     private function get_file_options() {
-        $fileoptions = array('subdirs'=>1,
-                                'maxbytes'=>$this->get_config('maxsubmissionsizebytes'),
-                                'maxfiles'=>$this->get_config('maxfilesubmissions'),
-                                'accepted_types'=>'*',
-                                'return_types'=>FILE_INTERNAL);
+        $fileoptions = array('subdirs' => 1,
+                                'maxbytes' => $this->get_config('maxsubmissionsizebytes'),
+                                'maxfiles' => $this->get_config('maxfilesubmissions'),
+                                'accepted_types' => $this->get_configured_typesets(),
+                                'return_types' => (FILE_INTERNAL | FILE_CONTROLLED_LINK));
+        if ($fileoptions['maxbytes'] == 0) {
+            // Use module default.
+            $fileoptions['maxbytes'] = get_config('assignsubmission_file', 'maxbytes');
+        }
         return $fileoptions;
     }
 
@@ -143,6 +165,7 @@ class assign_submission_file extends assign_submission_plugin {
      * @return bool
      */
     public function get_form_elements($submission, MoodleQuickForm $mform, stdClass $data) {
+        global $OUTPUT;
 
         if ($this->get_config('maxfilesubmissions') <= 0) {
             return false;
@@ -171,7 +194,6 @@ class assign_submission_file extends assign_submission_plugin {
      * @return int
      */
     private function count_files($submissionid, $area) {
-
         $fs = get_file_storage();
         $files = $fs->get_area_files($this->assignment->get_context()->id,
                                      'assignsubmission_file',
@@ -230,6 +252,9 @@ class assign_submission_file extends assign_submission_plugin {
         if (!empty($submission->userid) && ($submission->userid != $USER->id)) {
             $params['relateduserid'] = $submission->userid;
         }
+        if ($this->assignment->is_blind_marking()) {
+            $params['anonymous'] = 1;
+        }
         $event = \assignsubmission_file\event\assessable_uploaded::create($params);
         $event->set_legacy_files($files);
         $event->trigger();
@@ -238,7 +263,7 @@ class assign_submission_file extends assign_submission_plugin {
         $groupid = 0;
         // Get the group name as other fields are not transcribed in the logs and this information is important.
         if (empty($submission->userid) && !empty($submission->groupid)) {
-            $groupname = $DB->get_field('groups', 'name', array('id' => $submission->groupid), '*', MUST_EXIST);
+            $groupname = $DB->get_field('groups', 'name', array('id' => $submission->groupid), MUST_EXIST);
             $groupid = $submission->groupid;
         } else {
             $params['relateduserid'] = $submission->userid;
@@ -282,6 +307,28 @@ class assign_submission_file extends assign_submission_plugin {
         }
     }
 
+    /**
+     * Remove files from this submission.
+     *
+     * @param stdClass $submission The submission
+     * @return boolean
+     */
+    public function remove(stdClass $submission) {
+        global $DB;
+        $fs = get_file_storage();
+
+        $fs->delete_area_files($this->assignment->get_context()->id,
+                               'assignsubmission_file',
+                               ASSIGNSUBMISSION_FILE_FILEAREA,
+                               $submission->id);
+
+        $currentsubmission = $this->get_file_submission($submission->id);
+        $currentsubmission->numfiles = 0;
+        $DB->update_record('assignsubmission_file', $currentsubmission);
+
+        return true;
+    }
+
     /**
      * Produce a list of files suitable for export that represent this feedback or submission
      *
@@ -301,7 +348,12 @@ class assign_submission_file extends assign_submission_plugin {
                                      false);
 
         foreach ($files as $file) {
-            $result[$file->get_filename()] = $file;
+            // Do we return the full folder path or just the file name?
+            if (isset($submission->exportfullpath) && $submission->exportfullpath == false) {
+                $result[$file->get_filename()] = $file;
+            } else {
+                $result[$file->get_filepath().$file->get_filename()] = $file;
+            }
         }
         return $result;
     }
@@ -476,6 +528,28 @@ class assign_submission_file extends assign_submission_plugin {
         return $this->count_files($submission->id, ASSIGNSUBMISSION_FILE_FILEAREA) == 0;
     }
 
+    /**
+     * Determine if a submission is empty
+     *
+     * This is distinct from is_empty in that it is intended to be used to
+     * determine if a submission made before saving is empty.
+     *
+     * @param stdClass $data The submission data
+     * @return bool
+     */
+    public function submission_is_empty(stdClass $data) {
+        global $USER;
+        $fs = get_file_storage();
+        // Get a count of all the draft files, excluding any directories.
+        $files = $fs->get_area_files(context_user::instance($USER->id)->id,
+                                     'user',
+                                     'draft',
+                                     $data->files_filemanager,
+                                     'id',
+                                     false);
+        return count($files) == 0;
+    }
+
     /**
      * Get file areas returns a list of areas this plugin stores files
      * @return array - An array of fileareas (keys) and descriptions (values)
@@ -525,8 +599,42 @@ class assign_submission_file extends assign_submission_plugin {
         return array(
             'files_filemanager' => new external_value(
                 PARAM_INT,
-                'The id of a draft area containing files for this submission.'
+                'The id of a draft area containing files for this submission.',
+                VALUE_OPTIONAL
             )
         );
     }
+
+    /**
+     * Return the plugin configs for external functions.
+     *
+     * @return array the list of settings
+     * @since Moodle 3.2
+     */
+    public function get_config_for_external() {
+        global $CFG;
+
+        $configs = $this->get_config();
+
+        // Get a size in bytes.
+        if ($configs->maxsubmissionsizebytes == 0) {
+            $configs->maxsubmissionsizebytes = get_max_upload_file_size($CFG->maxbytes, $this->assignment->get_course()->maxbytes,
+                                                                        get_config('assignsubmission_file', 'maxbytes'));
+        }
+        return (array) $configs;
+    }
+
+    /**
+     * Get the type sets configured for this assignment.
+     *
+     * @return array('groupname', 'mime/type', ...)
+     */
+    private function get_configured_typesets() {
+        $typeslist = (string)$this->get_config('filetypeslist');
+
+        $util = new \core_form\filetypes_util();
+        $sets = $util->normalize_file_types($typeslist);
+
+        return $sets;
+    }
 }
